/*
        pravidla:

		1s=1000ms
		Uint32 SDL_GetTicks() vracia pocet ms od startu SDL,
		po zhruba 49 dnoch dojde k prekroceniu 32bit limitu
		a citac sa vynuluje
	uzitocne vzorce:

		rychlost		v=s/t [m/s]
					v=at [m/s]
		zrychlenie		a=s/t2 [m/s2]
*/
// hlavickove subory *******************************************************
// GL a GLU kniznica
#include <GL/gl.h>
#include <GL/glu.h>
// funkcie SDL kniznice
#include <SDL/SDL.h>

#include "OGLcamera.h"

// konstruktor *************************************************************
COGLcamera::COGLcamera()
{
 // naplnenie matice kamery pociatocnymi hodnotami
 for (t_a=0;t_a<16;t_a++) matrix[t_a]=0.0f;
 matrix[0]=1.0f; matrix[5]=1.0f; matrix[10]=1.0f; matrix[15]=1.0f;
 // nastavenie parametrov kamery
 posx=0.0f; posy=0.0f; posz=-15.0f;
 movx=0.0f; movy=0.0f; movz=0.0f;
 angx=0.0f; angy=0.0f; angz=0.0f;
 rotx=0.0f; roty=0.0f; rotz=0.0f;
 oldx=0.0f; oldy=0.0f; oldz=0.0f;
 targetx=0.0f; targety=0.0f; targetz=0.0f;
 speed=0.0f; slide=0.0f; alt=0.0f; acceleration=2.0f;
 falldown=0.01f;
 movetype=1;
 viewdist=1024.0f;
 FOV=60.0f;
 // nastavenie faktorov pre prepocet posunu a rotacie
 rotx_factor=4.0f; roty_factor=4.0f; rotz_factor=4.0f;
 movx_factor=10.0f; movy_factor=10.0f; movz_factor=10.0f;
 // pociatocne zresetovanie posunu
 goForward=0; goBack=0; goUp=0; goDown=0; goRight=0; goLeft=0;
 // prvotne nacitenie casoveho okamihu
 oldTime=SDL_GetTicks();

 ogl_fps=0.0f;
 ogl_frames=0.0f;
}

// destruktor **************************************************************
COGLcamera::~COGLcamera()
{

}

// inicializacia kamery a OpenGL kresliacej plochy *************************
void COGLcamera::cam_init(GLfloat w,GLfloat h)
{
 glMatrixMode(GL_PROJECTION); // vyber projekcnej matice
 glLoadIdentity();            // zresetovanie projekcnej matice

 if (h==0.0f) h=1.0f;
 gluPerspective(FOV,w/h,0.01f,viewdist); // nastavenie perspektivneho premietania
 // FOV - zorny uhol kamery, w/h - aspect je pomer vysky a sirky zobrazovanej plochy
 // 0.01f - minimalna viditelna vzdialenost, vievdist - maximalna viditelna vzdialenost
 glViewport(0,0,(GLuint) w,(GLuint) h);  // nastavenie viditelnej plochy

 // TRANSFORMUJEME SCENU NIE KAMERU!
 glMatrixMode(GL_MODELVIEW); // vyber modelovej matice
 glLoadIdentity();           // zresetovanie modelovej matice

 cam_update();

 scr_w=w; scr_h=h;
}

// aktualizacia nastaveni kamery *******************************************
void COGLcamera::cam_update(void)
{
 // ulozenie povodnej polohy
 oldx=posx;
 oldy=posy;
 oldz=posz;

 // SPECIALNY PRISTUP K ROTACII A NASLEDNEMU POSUNU KAMERY!
 glMatrixMode(GL_MODELVIEW);
 glLoadMatrixf(matrix);

 // ROTACIA
 glRotatef(rotx,matrix[0],matrix[4],matrix[8]);
 switch (movetype) {
  case 0:  // chodza
   glRotatef(roty,0.0f,1.0f,0.0f);
   break;
  case 1:  // let
   glRotatef(roty,matrix[1],matrix[5],matrix[9]);
   break;
  default: // dalsie typy pohybu...
   break;
 }
 glRotatef(rotz,matrix[2],matrix[6],matrix[10]);
 // nastavenie noveho otocenia
 angx+=rotx;
 angy+=roty;
 angz+=rotz;

 glGetFloatv(GL_MODELVIEW_MATRIX,matrix);

 // POSUN
 // nastavenie novej polohy
 posx+=matrix[0]*movx+matrix[1]*movy-matrix[2]*movz;
 posy+=matrix[4]*movx+matrix[5]*movy-matrix[6]*movz;
 posz+=matrix[8]*movx+matrix[9]*movy-matrix[10]*movz;
 // nastavenie smeru kamery
 targetx=posx+matrix[2];
 targety=posy+matrix[6];
 targetz=posz+matrix[10];
}

// posun kamery ************************************************************
void COGLcamera::cam_move(void)
{
 actTime=SDL_GetTicks();
 difTime=((GLfloat) (actTime-oldTime))/1000.0f;
 oldTime=actTime;

 // vypis casoveho okamihu uplynuteho medzi vykreslenim
 // dvoch scen
 //fprintf(stderr,"%f\n",difTime);

 if (goForward) speed=acceleration*difTime;
 else if (goBack) speed=-acceleration*difTime;
      else speed=0.0f;
 if (goRight) slide=acceleration*difTime;
 else if (goLeft) slide=-acceleration*difTime;
      else slide=0.0f;
 if (goUp) alt=acceleration*difTime;
 else if (goDown) alt=-acceleration*difTime;
      else alt=0.0f;

 movz=-speed*movz_factor;
 movy=-alt*movy_factor;
 movx=-slide*movx_factor;

 // zrusenie dalsieho mozneho posunu alebo rotacie
 goForward=0; goBack=0; goUp=0; goDown=0; goRight=0; goLeft=0;

 // pocitadlo FPS
 ogl_frames++;
 if (difTime>=0.01f) {
  ogl_fps=ogl_frames/difTime;
  ogl_frames=0.0f;
 }
}

// rotacia kamery **********************************************************
void COGLcamera::cam_rotate(GLfloat x,GLfloat y,GLfloat z)
{
 rotx=x/rotx_factor;
 roty=y/roty_factor;
 rotz=z/rotz_factor;
}

